
#ifndef INCLUDE_NAKEN_GLOBALS_H
#define INCLUDE_NAKEN_GLOBALS_H

/* ALTHOUGH NAKEN CHAT IS GPL, PLEASE BE KIND AND GIVE ME CREDIT FOR MY WORK. */
#define VERSION "Naken Chat 2.22 - February 1, 2011"
#define COPYRIGHT "Copyright 1998-2011 Michael Kohn"
/* PLEASE DON'T CHANGE THESE LINES */

#define MAXCONN 100
#define MAXCHAN 20 
#define MAX_USER_THREADS 4 
#define MAINCHANNEL "Main"
#define DEFAULT_PORT 6666
#define GC_TIME 300 
#define MAX_USER_NAME_LENGTH 17 
#define PASSWORD_LENGTH 20 
#define EMOTE_CHAR '%'
#define BAD_WORD_BUFFER_SIZE 1024
#define BANS_BUFFER_SIZE 4096
#define ALLOWS_BUFFER_SIZE 4096
#define NO_OWNER_BUFFER_SIZE 2048
#define WHO_BUFFER_SIZE 250
#define MAX_MESSAGE_SIZE 512
#define CHANNEL_NAME_LEN 20
#define HIDDEN_CAVES_START 1000
#define HIDDEN_CAVES_END 2000

#define CAN_BE_NOTHING 1
#define NOT_USERID 2

#ifdef WINDOWS
#define CONFIGDIR1 "c:\\Program Files\\nakenchat\\"
#define CONFIGDIR2 "c:\\nakenchat\\"
#else
#define CONFIGDIR1 "/usr/local/etc/"
#define CONFIGDIR2 "/etc/"
#endif

#ifdef WINDOWS
#define SHUT_RDWR SD_BOTH
#endif

#ifndef LOW_MEM_GAGS
#define IS_GAGGED(USER_ID, GAG_ID) (users[USER_ID]->gags[GAG_ID]!=0)
#define NOT_GAGGED(USER_ID, GAG_ID) (users[USER_ID]->gags[GAG_ID]==0)
#define TOGGLE_GAG(USER_ID, GAG_ID) users[USER_ID]->gags[GAG_ID]^=1;
#else
#define IS_GAGGED(USER_ID, GAG_ID) ((users[USER_ID]->gags[GAG_ID/8]&(1<<(GAG_ID%8)))!=0)
#define NOT_GAGGED(USER_ID, GAG_ID) ((users[USER_ID]->gags[GAG_ID/8]&(1<<(GAG_ID%8)))==0)
// #define TOGGLE_GAG(USER_ID, GAG_ID) users[USER_ID]->gags[GAG_ID/8]=(unsigned char)((int)users[ID]->gags[GAG_ID/8]^(1<<GAG_ID%8))
// #define TOGGLE_GAG(USER_ID, GAG_ID) users[USER_ID]->gags[GAG_ID/8]=(users[ID]->gags[GAG_ID/8]^(1<<GAG_ID%8))
#define TOGGLE_GAG(USER_ID, GAG_ID) users[USER_ID]->gags[GAG_ID/8]^=(1<<GAG_ID%8)
#endif

#include "naken_user.h"

#ifdef PLUGINS
#include "naken_plugin.h"

#ifdef WINDOWS
#define dlsym GetProcAddress
#define dlopen(libname,x) LoadLibrary(libname)
#define dlclose(libhandle) FreeLibrary(libhandle)
#endif

typedef int (*naken_init_t)(struct naken_context_t *naken_context);
typedef int (*naken_dot_command_t)(int ID, const char *text);
typedef int (*naken_public_text_t)(int ID, char *text);
typedef int (*naken_output_filter_t)(char *text);

struct naken_plugins_t
{
  /* These functions could be separated into other linked lists to
     save CPU cycles... that way if a function isn't implemented
     it won't be in the list... */

  // naken_init_t naken_init;
  naken_dot_command_t naken_dot_command;
  naken_public_text_t naken_public_text;
  naken_output_filter_t naken_output_filter;
#ifndef WINDOWS
  void *dlhandle;
#else
  HMODULE dlhandle;
#endif
  struct naken_plugins_t *next_plugin;
};

struct naken_plugins_t *naken_plugins;
struct naken_context_t naken_context;
#endif

extern int sockfd,bancount,allowcount,badwordcount,port;
extern struct User nulluser; 
extern struct User **users; 
extern char channels[MAXCHAN][CHANNEL_NAME_LEN];
extern int channelop[MAXCHAN];
extern char noowner[NO_OWNER_BUFFER_SIZE];
extern int cflags[MAXCHAN];
extern int international,debug,censor;
extern unsigned int uptime;
extern unsigned int serverflags;
extern int register_level;
extern char sysoppw[15];
extern char bans[BANS_BUFFER_SIZE];
extern char allows[ALLOWS_BUFFER_SIZE];
extern char badword[BAD_WORD_BUFFER_SIZE];
extern char welcome[2048];
extern char logfile[256];
#ifdef WHOFILE
extern char whofile[256];
#ifdef WINDOWS
extern CRITICAL_SECTION win_mutex;
#else
extern pthread_mutex_t pthread_mutex;
extern pthread_mutexattr_t pthread_mutexattr;
#endif
#endif
extern char passwd[256];
extern char configdir[256];
extern int welcomelen;
extern int logging;
extern int login_speed;
extern int maxconn;
extern int minconn;
extern time_t max_idle_time;
extern FILE *logfp;

#define UFLAG_ISSET(id,flag) ((users[id]->uflags&flag)==flag)
#define UFLAG_NOTSET(id,flag) ((users[id]->uflags&flag)==0)
#define UFLAG_SET(id,flag) users[id]->uflags|=flag
#define UFLAG_TOGGLE(id,flag) users[id]->uflags^=flag

#define SERVER_ISSET(flag) ((serverflags&flag)==flag)
#define SERVER_NOTSET(flag) ((serverflags&flag)==0)
#define SERVER_SET(flag) serverflags|=flag
#define SERVER_TOGGLE(flag) serverflags^=flag

/*

Server Flags

0: channel disallow   (1)
1: host lookups       (2)
2: show ips           (4)
3: no dup ips         (8)
4: elite mode         (16)
5: force password     (32)
6: force names        (64)
7: no duplicate names (128)
8: no name changing   (256)
9: kick dups          (512)
10: allow register    (1024)
*/

#define SERVER_CHANNEL_DISALLOW 1
#define SERVER_HOST_LOOKUPS 2
#define SERVER_SHOW_IPS 4
#define SERVER_NO_DUP_IPS 8
#define SERVER_ELITE_MODE 16
#define SERVER_FORCE_PASSWORD 32
#define SERVER_FORCE_NAMES 64
#define SERVER_NO_DUP_NAMES 128
#define SERVER_NO_NAME_CHANGING 256
#define SERVER_KICK_DUPS 512
#define SERVER_NAME_REGISTER 1024


/*

User Flags
0: Beeps
1: Hilites
2: Squelched
3: Hush Yells
4: Echos
5: Time Stamping
6: System Messages
7: Hidden Sysop
8: Client Userlist
9: User already changed their name

*/

#define USER_BEEPS 1
#define USER_HILITES 2
#define USER_SQUELCHED 4
#define USER_HUSH_YELLS 8
#define USER_ECHOS 16
#define USER_TIME_STAMPS 32
#define USER_SYSTEM_MESSAGES 64
#define USER_HIDDEN_SYSOP 128
#define USER_USERLIST 256
#define USER_CHANGED_NAME 512

#endif

