#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#ifndef WINDOWS
#include <pthread.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <signal.h>
#include <netdb.h>
#endif
#include <errno.h>
#include <time.h>
#include <string.h>
#ifdef WINDOWS
#include <windows.h>
#include <process.h>
#include <winsock.h>
#endif

#include "language.h"
#include "globals.h"
#include "functions.h"

int sockfd,bancount,allowcount,badwordcount,port;
struct User **users;
struct User nulluser;
char channels[MAXCHAN][CHANNEL_NAME_LEN];
int channelop[MAXCHAN];
char noowner[NO_OWNER_BUFFER_SIZE];
int cflags[MAXCHAN];
int international,debug,censor;
unsigned int uptime;
char sysoppw[15];
unsigned int serverflags;
int register_level=-1;
char bans[BANS_BUFFER_SIZE];
char allows[ALLOWS_BUFFER_SIZE];
char badword[BAD_WORD_BUFFER_SIZE];
char welcome[2048];
char logfile[256];
#ifdef WHOFILE
char whofile[256];
#ifdef WINDOWS
CRITICAL_SECTION win_mutex;
#else
pthread_mutex_t pthread_mutex;
pthread_mutexattr_t pthread_mutex_attr;
#endif
#endif
char passwd[256];
char configdir[256];
int welcomelen;
int logging;
int login_speed=0;
int maxconn;
int minconn;
int gag_size;
time_t max_idle_time=0;
FILE *logfp;
int socket_errors=0;
#ifdef PLUGINS
struct naken_plugins_t *naken_plugins=0;
struct naken_context_t naken_context;
#endif

static int init(int socketid, struct sockaddr_in *cli_addr, socklen_t clilen)
{
int k,r,t,ID;
char tempString[200];
struct hostent *my_hostent=NULL;
unsigned char *address=0;
char *fullmessage={ SORRY_FULL };
char *toomanyconnections={ TOO_MANY_CONNECTIONS };
char *allowptr;
char *banptr;

  if (SERVER_ISSET(SERVER_HOST_LOOKUPS))
  { my_hostent=gethostbyaddr((char *)&(cli_addr->sin_addr.s_addr),4,AF_INET); }

  for (r=0; r<maxconn; r++)
  { 
    if (users[r]->inuse==0) 
    { break; } 
  }

  if (r>=maxconn)
  {
#ifdef DEBUG
    if (debug==1) printf("Chat server full\n");
#endif
    send(socketid,fullmessage,strlen(fullmessage),0); 
    socketdie(socketid);
    return -1;
  }

  if (users[r]==&nulluser)
  { 
#ifdef DEBUG
    if (debug==1) printf("Allocing new user %d\n",r);
#endif
    users[r]=(struct User *)malloc(sizeof(struct User)); 
    memset(users[r],0,sizeof(struct User));
    users[r]->gags=malloc(gag_size); 
  }

#ifdef DEBUG
  if (debug==1)
  {
    printf("searched for open User: found %d\n",r);
    fflush(stdout);
  }
#endif

  if (my_hostent!=NULL && SERVER_ISSET(SERVER_HOST_LOOKUPS))
  { safestrncpy(users[r]->location,my_hostent->h_name,61); }
    else
  { 
    address=(unsigned char*)&cli_addr->sin_addr.s_addr;
    sprintf(users[r]->location,"%d.%d.%d.%d",address[0],address[1],address[2],address[3]);
  }

#ifdef DEBUG
  if (debug==1)
  {
    printf("Connected from location: %s\n",users[r]->location);
    fflush(stdout);
  }
#endif

  if (SERVER_ISSET(SERVER_NO_DUP_IPS))
  {
    for (t=0; t<maxconn; t++)
    { 
      if (t==r) continue;

      if (users[t]->inuse!=0) 
      {
        if (strcmp(users[t]->location,users[r]->location)==0)
        {
#ifdef DEBUG
          if (debug==1) printf("Too many connections from this IP\n");
#endif
          send(socketid,toomanyconnections,strlen(toomanyconnections),0); 
          socketdie(socketid);
          return -1;
        }
      } 
    }
  }

  if (login_speed!=0)
  {
    k=time(NULL);
    for (t=0; t<maxconn; t++)
    { 
      if (t==r) continue;

      if (users[t]->inuse!=0) 
      {
        if ((k-users[t]->logontime)<login_speed && strcmp(users[t]->location,users[r]->location)==0)
        {
#ifdef DEBUG
          if (debug==1) printf("Fast login - user kicked\n");
#endif
          send(socketid,toomanyconnections,strlen(toomanyconnections),0); 
          socketdie(socketid);
          return -1;
        }
      } 
    }
  }

  banptr=bans;
  for (t=0; t<bancount; t++)
  { 
    if (strcasestr(users[r]->location,banptr)!=NULL)
    {
      send(socketid,BANNED,strlen(BANNED),0);
#ifdef DEBUG
      if (debug==1)
      { 
        printf("This site was banned(%d):\n",socketdie(socketid)); 
        fflush(stdout); 
      }
#endif

      socketdie(socketid);
      return -1;
    }

    banptr=banptr+strlen(banptr)+1;
  }

  if (allowcount!=0 && t==bancount)
  {
    allowptr=allows;
    for (t=0; t<allowcount; t++)
    {
      if (strcasestr(users[r]->location,allowptr)!=NULL) break;
      allowptr=allowptr+strlen(allowptr)+1;
    }

    if (t==allowcount) 
    {
      send(socketid,REFUSED,strlen(REFUSED),0);
#ifdef DEBUG
      if (debug==1)
      { 
        printf("This site was banned(%d):\n",socketdie(socketid)); 
        fflush(stdout); 
      }
#endif

      socketdie(socketid);
      return -1;
    }
  }

/*
  k=0;

  if (SERVER_ISSET(SERVER_NO_DUP_IPS))
  {
    for (k=0; k<maxconn; k++)
    {
      if (users[k]->inuse!=1) continue;
      if (strcmp(users[r]->location,users[k]->location)==0)
      { k=-1; break; }
    }
  }
*/

  users[r]->ID=r;
  users[r]->socketid=socketid;
  ID=r;

  users[ID]->spam=0;
  users[ID]->channel=0;
  users[ID]->user_level=0;
  users[ID]->uflags=USER_ECHOS;
  users[ID]->stamptime=0;
  users[ID]->lastp=-1;
  users[ID]->ptr=0;
  users[ID]->blen=0;
  users[ID]->bptr=0;
  users[ID]->emote_char=EMOTE_CHAR;
  users[ID]->logontime=time(NULL);
  users[ID]->idletime=time(NULL);
#ifdef REALIDLETIME
  users[ID]->real_idletime=time(NULL);
#endif
  safestrncpy(users[ID]->username,users[ID]->location,MAX_USER_NAME_LENGTH);
  memset(users[ID]->gags,0,gag_size);

  users[ID]->socketfd=fdopen(socketid,"rb+");

  if (users[ID]->socketfd==0) printf("Problem opening file desc.\n\n");

#ifndef WINDOWS
  fcntl(users[ID]->socketid, F_SETFL, O_NONBLOCK);
#else
  u_long arg=1L;
  ioctlsocket(users[ID]->socketid, FIONBIO, (u_long FAR *)&arg);
#endif

/* hmmm 

  if (setsockopt(sockfd,SOL_SOCKET,SO_RCVLOWAT,&sopt,sizeof(sopt)))
  { printf("socket options SO_RCVLOWAT can't be set.\n"); }
  if (setsockopt(sockfd,SOL_SOCKET,SO_SNDLOWAT,&sopt,sizeof(sopt)))
  { printf("socket options SO_SNDLOWAT can't be set.\n"); }
*/
 
  t=0;
  while((users[ID]->username[t]!=0 && users[ID]->username[t]!='.') && t<16) t++;
  users[ID]->username[t]=0; 

  if (SERVER_ISSET(SERVER_NO_DUP_NAMES))
  {
    if (checkname_dup(users[ID]->username,ID)!=-1)
    { strcpy(users[ID]->username,"unnamed"); }
  }

#ifdef PASSWD
  if (passwd[0]!=0)
  {
    if (passwd_check(users[ID]->username,-1)==1)
    { strcpy(users[ID]->username,"unnamed"); }
  }
#endif

  for (r=0; r<gag_size; r++)
  { users[ID]->gags[r]=0; }

  send(users[ID]->socketid,welcome,welcomelen,0);

  if (SERVER_ISSET(SERVER_FORCE_NAMES))
  {
/*
    sprintf(tempString,NEED_TO_LOGIN,ID,users[ID]->location);
    message(ID,tempString,0);
*/

    users[ID]->inuse=100;
  }
    else
  {
    users[ID]->inuse=1;

    sprintf(tempString,YOU_LOGIN,ID,users[ID]->location);
    message(ID,tempString,0);

    if (SERVER_ISSET(SERVER_SHOW_IPS))
    { sprintf(tempString,SOMEONE_LOGIN1,ID,users[ID]->location); }
      else
    { sprintf(tempString,SOMEONE_LOGIN2,ID); }

    sendallbut(ID,tempString);
    update_list(ID,1);
  }

/*
  sprintf(tempString,"+[%d]%s\r\n",ID,users[ID]->username);
  for (r=0; r<maxconn; r++)
  {
    if (users[r]->inuse==1 && UFLAG_ISSET(r, USER_USER_LIST))
    { message(r,tempString,0); }
  }
*/

#ifdef WHOFILE
  if (whofile[0]!=0) write_whofile();
#endif

  return 0;
}

void kill_user(int ID)
{
int r;
char spa[80];

  if (users[ID]->inuse==5) return;
  if (users[ID]->inuse==1) users[ID]->inuse=5;

#ifdef DEBUG
  if (debug==1)
  {
    printf("Getting ready to close socket.\n");
    if (users[ID]->inuse!=4)
      printf("Socket Close Return value: %d:\n",socketdie(users[ID]->socketid));
  }
    else
#endif
  {
    if (users[ID]->inuse!=4) socketdie(users[ID]->socketid);
  }

  users[ID]->socketid=0;

  if (users[ID]->channel!=0) channel(ID,"0");

  for (r=0; r<MAXCHAN; r++) 
  { if (channelop[r]==ID) channelop[r]=-1; }
/*
  curr=users[ID]->channel;
  if (curr!=0)
  {
    if (channelop[curr]==ID) channelop[curr]=-1;
    for (r=0; r<maxconn; r++)
    {
      if (users[r]->inuse==1)
      {
        if (users[r]->channel==curr && r!=ID) r=maxconn+6;
      }
    }

    if (r<maxconn)
    { channels[curr][0]=0; channelop[curr]=-1; cflags[curr]=0; }
  }
*/

  for (r=0; r<maxconn; r++)
  {
    if (users[r]->inuse==1)
    {
      if (IS_GAGGED(r,ID))
      {
        TOGGLE_GAG(r,ID);
      }
    }
  }
  for (r=0; r<maxconn; r++) if (users[r]->inuse==1 && users[r]->lastp==ID) users[r]->lastp=-1;

#ifdef DEBUG
  if (debug==1) printf("Connection dead: %d\n",ID);
  fflush(stdout);
#endif

  if (users[ID]->inuse!=100)
  {
    users[ID]->inuse=0;
    sprintf(spa,JUST_LEFT,ID,users[ID]->username);

    if (SERVER_NOTSET(SERVER_CHANNEL_DISALLOW))
    { sendallbut(ID,spa); }
      else
    { sendchanbut(ID,users[ID]->channel,spa); }

    update_list(ID,0);
  }
    else
  { users[ID]->inuse=0; }

  users[ID]->idletime=time(NULL);
#ifdef REALIDLETIME
  users[ID]->real_idletime=time(NULL);
#endif

#ifdef WHOFILE
  if (whofile[0]!=0) write_whofile();
#endif
}

int main(int argc,char *argv[])
{
struct sockaddr_in serv_addr;
int r,sopt=1;
struct linger mylinger;
int newsockfd;
socklen_t clilen;
struct sockaddr_in cli_addr;
int threads[MAX_USER_THREADS];
#ifndef WINDOWS
pthread_t pid;
#endif
#ifdef WINDOWS
WSADATA wsaData;
WORD wVersionRequested=MAKEWORD(1,1);
int Win32isStupid;

  Win32isStupid=WSAStartup(wVersionRequested, &wsaData);
  if (Win32isStupid)  /* Shouldn't this always be true? :) */
  {
    printf("Winsock can't start.\n");
    exit(1);
  }

/*
  printf("Max sockets: %d\n",wsaData.iMaxSockets);
  printf("Description: %s\n",wsaData.szDescription);
*/
#endif

  maxconn=MAXCONN;
  minconn=10;
  max_idle_time=0;
  setup(argc,argv);

#ifndef WINDOWS
  if (debug==0)
  {
    r=fork();

    if (r==-1)
    { perror("Error: Could not fork\n"); exit(3); }
      else
    if (r==0)
    {
      close(STDIN_FILENO);
      close(STDERR_FILENO);

      if (setsid()==-1)
      { exit(4); }
    }
      else
    { return 0; }
  }
#endif

#ifdef DEBUG
  if (debug==1)
  {
    printf("minconn: %d\n",minconn);
    printf("maxconn: %d\n",maxconn);
  }
#endif

  if ((sockfd=socket(AF_INET,SOCK_STREAM,0))<0)
  { printf("Can't open socket.\n"); exit(1); }

  /* bzero((char*)&serv_addr,sizeof(serv_addr)); */
  memset((char*)&serv_addr,0,sizeof(serv_addr));
  serv_addr.sin_family=AF_INET;
  serv_addr.sin_addr.s_addr=htonl(INADDR_ANY);
  serv_addr.sin_port=htons(port);

  mylinger.l_onoff=0;
  mylinger.l_linger=1;

#ifndef WINDOWS
  if (setsockopt(sockfd,SOL_SOCKET,SO_REUSEADDR,&sopt,sizeof(sopt)))
  { printf("socket options REUSERADDR can't be set.\n"); }
  if (setsockopt(sockfd,SOL_SOCKET,SO_LINGER,&mylinger,sizeof(struct linger)))
  { printf("socket options LINGER can't be set.\n"); }
#ifdef KEEPALIVE
  if (setsockopt(sockfd,SOL_SOCKET,SO_KEEPALIVE,&sopt,sizeof(sopt)))
  { printf("socket options KEEPALIVE can't be set.\n"); }
#endif
#else
  if (setsockopt(sockfd,SOL_SOCKET,SO_REUSEADDR,(char *)&sopt,sizeof(sopt)))
  { printf("socket options REUSERADDR can't be set.\n"); }
  if (setsockopt(sockfd,SOL_SOCKET,SO_LINGER,(char *)&mylinger,sizeof(struct linger)))
  { printf("socket options LINGER can't be set.\n"); }
#ifdef KEEPALIVE
  if (setsockopt(sockfd,SOL_SOCKET,SO_KEEPALIVE,(char *)&sopt,sizeof(sopt)))
  { printf("socket options KEEPALIVE can't be set.\n"); }
#endif
#endif

  if(bind(sockfd,(struct sockaddr *)&serv_addr,sizeof(serv_addr))<0)
  { printf("server can't bind.\n"); exit(1); }

  listen(sockfd,5);
#ifndef WINDOWS
  setsignals();
#endif

/* YOU MAY NOT EDIT THE NEXT LINE OF CODE */
  printf("\n"VERSION"\n"COPYRIGHT"\n\n");
/* NAKEN CHAT IS NOT GPL */

  printf("Started on port: %d\n",port);
  fflush(stdout);

  if (debug==0) close(STDOUT_FILENO);

  nulluser.inuse=0;
  nulluser.idletime=-1;
#ifdef REALIDLETIME
  nulluser.real_idletime=-1;
#endif
  users=malloc(sizeof(struct User*)*maxconn);
#ifdef PLUGINS
  naken_context.users=users;
#endif

  for (r=0; r<maxconn; r++)
  {
    users[r]=&nulluser;
  }

#ifndef LOW_MEM_GAGS
  gag_size=maxconn;
#else
  gag_size=maxconn/8;
  if ((maxconn%8)!=0) gag_size++;
#endif

  for (r=0; r<minconn; r++)
  {
    users[r]=(struct User *)malloc(sizeof(struct User));
    memset(users[r],0,sizeof(struct User));
    users[r]->gags=malloc(gag_size); 
    memset(users[r]->gags,0,gag_size);
    users[r]->inuse=0;
  }

#ifdef WHOFILE
#ifndef WINDOWS
  pthread_mutexattr_init(&pthread_mutex_attr);
  pthread_mutex_init(&pthread_mutex, &pthread_mutex_attr);
#else
  InitializeCriticalSection(&win_mutex);
#endif
#endif

  for (r=0; r<MAX_USER_THREADS; r++)
  {
    threads[r]=r;
#ifndef WINDOWS
    pthread_create(&pid,NULL,(void*)chat,&threads[r]);
#else
    _beginthread((void*)chat,0,&threads[r]);
#endif
  }

  /* chat(); */

  clilen=sizeof(cli_addr);

  while(1)
  {
    newsockfd=accept(sockfd,(struct sockaddr *)&cli_addr,&clilen);

    if (newsockfd<0)
    {
#ifdef DEBUG
      if (debug==1) { printf("server accept error.\n"); }
#endif
      socket_errors++;
      if (socket_errors==5) break;
    }
      else
    { 
#ifdef DEBUG
      if (debug==1)
      {
        printf("New socket accepted\n");
        fflush(stdout);
      }
#endif
      init(newsockfd,&cli_addr,clilen);
      socket_errors=0;
    }
  }

  return 0;
}

