#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#ifndef WINDOWS
#include <sys/socket.h>
#include <netinet/in.h>
#include <pwd.h>
#include <grp.h>
#ifdef PLUGINS
#include <dlfcn.h>
#endif
#endif
#include <time.h>
#include <ctype.h>
#ifdef WINDOWS
#include <windows.h>
#endif

#include "globals.h"
#include "functions.h"
#include "language.h"

#ifdef PLUGINS
static int plugins_read=0;   // Don't allow reset of plugins.
#endif

extern char help_message[sizeof(HELP)+sizeof(CHAN_HELP)+2];

static void addtowelcome(char *s)
{
int t,l;

  l=strlen(s);

  for (t=0; t<l; t++)
  { 
    if (s[t]=='\n' || s[t]=='\r') break;
    if (s[t]<32) continue;
    welcome[welcomelen++]=s[t];
  }
  
  welcome[welcomelen++]='\r';
  welcome[welcomelen++]='\n';
  welcome[welcomelen]=0;
}

void read_config()
{
FILE *fp;
int w,t;
char name[100];
char *param;
char *badwordptr;
char *allowptr;
#ifdef PLUGINS
naken_init_t naken_plugin_init;
struct naken_plugins_t naken_plugin={0};
struct naken_plugins_t *next_naken_plugin;
#endif
#ifndef WINDOWS
struct passwd *pw;
struct group *gr;
#endif

  allowcount=0;
  allowptr=allows;
  bancount=0;
  badwordcount=0;
  badwordptr=badword;
  welcomelen=0;
#ifdef WHOFILE
  whofile[0]=0;
#endif
  noowner[0]=0;

  fp=fopen(configdir,"r");
 
  if (fp==0)
  { 
    printf("Couldn't open %s.  Trying "CONFIGDIR2"nakenchat.conf\n",configdir);

    strcpy(configdir,CONFIGDIR2"nakenchat.conf");
    fp=fopen(configdir,"r");

    if (fp==0)
    {
      printf("Couldn't open %s. Trying ./nakenchat\n",configdir);

      strcpy(configdir,"./nakenchat.conf");
      fp=fopen(configdir,"r");

      if (fp==0)
      {
        printf("No config file found.  Going with defaults.\n");
      }
    } 
  }

  if (fp!=0)
  {
    printf("Config File: %s\n",configdir);

    while(1)
    {
      param=0;
      t=0;
      
      while((w=getc(fp))!=EOF)
      {
        if (w=='\n' || w=='\r') break;
        if (w==' ' || w=='\t')
        {
          if (t==0) continue;
          if (param==0)
          {
            name[t++]=0;
            param=name+t;
            continue;
          }
            else
          if (param==name+t)
          {
            param++;
          }
        }

        name[t++]=w;
      }

      if (w==EOF && t==0) break;
      if (t==0 || name[0]=='#') continue;

      name[t]=0;
      if (param==0) param=name+t;

      t=strlen(param);
      while(t>0 && (param[t-1]==' ' || param[t-1]=='\t'))
      { param[--t]=0; }

      /* printf("%s=%s;\n",name,param); */

      if (strcasecmp(name,"SYSOP_PASSWORD")==0)
      { safestrncpy(sysoppw,param,15); }
        else
      if (strcasecmp(name,"BAD_WORD")==0)
      {
        if (strlen(param)+badwordptr+1>=badword+BAD_WORD_BUFFER_SIZE)
        {
          printf("Exceeded %d bytes for bad words.  Cannot add word %s\n",BAD_WORD_BUFFER_SIZE,param);
        }
          else
        {
          strcpy(badwordptr,param);
          badwordptr=badwordptr+strlen(param)+1;
          badwordcount++;
        }
      }
        else
      if (strcasecmp(name,"ALLOW")==0)
      {
        if (strlen(param)+allowptr+1>=allows+ALLOWS_BUFFER_SIZE)
        {
          printf("Exceeded %d bytes for allows.  Cannot add site %s\n",ALLOWS_BUFFER_SIZE,param);
        }
          else
        {
          strcpy(allowptr,param);
          allowptr=allowptr+strlen(param)+1;
          allowcount++;
        }
      }
        else
      if (strcasecmp(name,"BAN")==0)
      {
        if (ban_address(param)==1)
        {
          printf("Exceeded %d bytes for bans.  Cannot add site %s\n",BANS_BUFFER_SIZE,param);
        }
      }
        else
      if (strcasecmp(name,"INTERNATIONAL")==0)
      { if (atoi(param)==1) international=1; }
        else
      if (strcasecmp(name,"PORT")==0)
      { port=atoi(param); }
        else
      if (strcasecmp(name,"HOSTLOOKUPS")==0)
      { if (atoi(param)==1) SERVER_SET(SERVER_HOST_LOOKUPS); }
        else
      if (strcasecmp(name,"SHOWIPS")==0)
      { if (atoi(param)==1) SERVER_SET(SERVER_SHOW_IPS); }
        else
      if (strcasecmp(name,"WELCOME")==0)
      { addtowelcome(param); }
        else
      if (strcasecmp(name,"NODUPIPS")==0)
      { if (atoi(param)==1) SERVER_SET(SERVER_NO_DUP_IPS); }
        else
      if (strcasecmp(name,"LOGFILE")==0)
      { strcpy(logfile,param); }
        else
      if (strcasecmp(name,"MAXCONN")==0)
      { maxconn=atoi(param); }
        else
      if (strcasecmp(name,"MINCONN")==0)
      { minconn=atoi(param); }
        else
      if (strcasecmp(name,"MAX_IDLE_TIME")==0)
      { max_idle_time=atoi(param)*60; }
        else
      if (strcasecmp(name,"PASSWD")==0)
      { strcpy(passwd,param); }
        else
      if (strcasecmp(name,"CENSOR")==0)
      { censor=atoi(param); }
        else
      if (strcasecmp(name,"CHANNEL_DISABLE")==0)
      { if (atoi(param)==1) SERVER_SET(SERVER_CHANNEL_DISALLOW); }
        else
      if (strcasecmp(name,"LOGGING")==0)
      { if (atoi(param)==1) logfile_on(-1); }
        else
      if (strcasecmp(name,"FORCE_PASSWD")==0)
      { if (atoi(param)==1) SERVER_SET(SERVER_FORCE_PASSWORD); }
        else
      if (strcasecmp(name,"FORCE_NAME")==0)
      { if (atoi(param)==1) SERVER_SET(SERVER_FORCE_NAMES); }
        else
      if (strcasecmp(name,"NODUP_NAMES")==0)
      { if (atoi(param)==1) SERVER_SET(SERVER_NO_DUP_NAMES); }
        else
      if (strcasecmp(name,"NO_NAME_CHANGE")==0)
      { if (atoi(param)==1) SERVER_SET(SERVER_NO_NAME_CHANGING); }
        else
      if (strcasecmp(name,"LOGIN_SPEED")==0)
      { login_speed=atoi(param); }
#ifdef WHOFILE
        else
      if (strcasecmp(name,"WHO_FILE")==0)
      { strcpy(whofile,param); }
#endif
        else
      if (strcasecmp(name,"KICK_DUPS")==0)
      { if (atoi(param)==1) SERVER_SET(SERVER_KICK_DUPS); }
        else
      if (strcasecmp(name,"RUN_AS")==0)
      { 
#ifndef WINDOWS
        t=0;
        while(param[t]!=':' && param[t]!=0) t++;

        if (param[t]==':')
        {
          param[t]=0;
          t++;
          gr=getgrnam(param+t);
          if (gr==0)
          { printf("No such group %s\n",param+t); }
            else
          if (setgid(gr->gr_gid)!=0)
          { printf("Couldn't set group %s\n",param+t); }
        }

        pw=getpwnam(param);

        if (pw==0)
        { printf("No such user %s\n",param); }
          else
        if (setuid(pw->pw_uid)!=0)
        { printf("Couldn't set user to %s\n",param); }
#else
        printf("RUN_AS is not supported on Windows\n");
#endif
      }
        else
      if (strcasecmp(name,"NO_OWNER")==0)
      {
        if (add_to_list(noowner, param, NO_OWNER_BUFFER_SIZE)!=0)
        {
          printf("Too many entries for NO_OWNER, failed on %s\n",param);
        }
      }
#ifdef PASSWD
        else
      if (strcasecmp(name,"PASSWD_LEVEL")==0)
      {
        register_level=atoi(param);
        if (register_level<-1) register_level=-1;
          else
        if (register_level>10) register_level=10;

        if (register_level>=0) SERVER_SET(SERVER_NAME_REGISTER);
      }
#endif
        else
      if (strcasecmp(name,"PLUGIN")==0)
      {
#ifdef PLUGINS
        naken_plugin.dlhandle=dlopen(param,RTLD_LAZY);
        if (naken_plugin.dlhandle==0)
        {
          printf("Couldn't open plugin %s\n",param);
          continue;
        }
        naken_plugin_init=(naken_init_t)dlsym(naken_plugin.dlhandle,"naken_init");
        if (naken_plugin_init!=0)
        {
          naken_plugin_init(&naken_context);
        }

        naken_plugin.naken_dot_command=(naken_dot_command_t)dlsym(naken_plugin.dlhandle,"naken_dot_command");
        naken_plugin.naken_public_text=(naken_public_text_t)dlsym(naken_plugin.dlhandle,"naken_public_text");
        naken_plugin.naken_output_filter=(naken_output_filter_t)dlsym(naken_plugin.dlhandle,"naken_output_filter");

        if (naken_plugins==0)
        {
          naken_plugins=(struct naken_plugins_t *)malloc(sizeof(struct naken_plugins_t));
          memcpy(naken_plugins,&naken_plugin,sizeof(naken_plugin));
        }
          else
        {
          next_naken_plugin=naken_plugins;
          while(next_naken_plugin->next_plugin!=0)
          { next_naken_plugin=next_naken_plugin->next_plugin; }
          naken_plugins->next_plugin=(struct naken_plugins_t *)malloc(sizeof(struct naken_plugins_t));
          memcpy(naken_plugins->next_plugin,&naken_plugin,sizeof(naken_plugin));
        }
#else
        printf("Config error: Plugins not enabled!\n");
#endif
      }
    } 

    fclose(fp);
  }

  if (welcomelen==0)
  {
    strcpy(welcome,">> Naken Chat - Copyright 1998-2009 Michael Kohn\r\n\r\n");
    welcomelen=strlen(welcome);
    return;
  }

  sprintf(help_message,HELP,EMOTE_CHAR);

  if (SERVER_NOTSET(SERVER_CHANNEL_DISALLOW))
  { strcat(help_message,CHAN_HELP); }
  /* { strcpy(&help_message[sizeof(HELP)-1],CHAN_HELP); } */

  strcat(help_message,"\r\n");

#ifdef PLUGINS
  plugins_read=1;
#endif
}

void setup(int argc, char *argv[])
{
char *allowptr;
char *banptr;
int r;

  port=DEFAULT_PORT;
  configdir[0]=0;

#ifdef PLUGINS
  /* naken_context.users=users; */
  naken_context.message=message;
  naken_context.sendpriv=sendpriv;
  naken_context.sendchan=sendchan;
  naken_context.sendchanbut=sendchanbut;
  naken_context.sendall=sendall;
  naken_context.sendallbut=sendallbut;
  naken_context.kill_user=kill_user;
#endif

  for (r=1; r<argc; r++)
  {
    if (strcasecmp(argv[r],"-f")==0) 
    { strcpy(configdir,argv[++r]); break; }
  }

  logfile[0]=0;
  passwd[0]=0;

  if (configdir[0]==0)
  {
     strcpy(configdir,CONFIGDIR1"nakenchat.conf");
  }

  strcpy(channels[0],MAINCHANNEL);
  strcpy(sysoppw,"password");
  uptime=time(NULL);
  srand(uptime);

  read_config();

  if (logfile[0]!=0) printf("Logfile: %s\n",logfile);

  printf("Currently banned sites:\n");
  banptr=bans;
  for (r=0; r<bancount; r++)
  {
    printf("%d: %s\n",r,banptr);
    banptr=banptr+strlen(banptr)+1;
  }

  printf("%d bans found\n\n",bancount);

  if (allowcount!=0)
  {
    printf("Only these sites may logon:\n");
    allowptr=allows;
    for (r=0; r<allowcount; r++)
    {
      printf("%d: %s\n",r,allowptr);
      allowptr=allowptr+strlen(allowptr)+1;
    }
  }

  for (r=1; r<argc; r++)
  {
    if (strcmp(argv[r],"-p")==0)
    { port=atoi(argv[++r]); }
      else
    if (strcasecmp(argv[r],"-d")==0) debug=1;
  }
}

